#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase


class FecTest(TestCase):

    def test_fake(self):
        self.assertEqual(2+2, 4)



import os
from django.conf import settings
from postgres_copy import CopyMapping
from fec_raw import get_download_directory
from string import ascii_letters, punctuation
from fec_raw.management.commands import FecCommand
from fec_raw.models import RawF3XFiling, RawF3PFiling, RawF24Filing
from fec_raw.models import RawContribution, RawIndependentExpenditure


class Command(FecCommand):
    help = "Load downloaded filings into database"
    DATA_DIR = get_download_directory()

    def add_arguments(self, parser):
        parser.add_argument(
            '--flush',
            action='store_true',
            dest='flush',
            default=False,
            help='Flush the database and start anew',
        )

    def handle(self, *args, **options):
        """
        For each of the filing types we support, loops through the downloaded
        CSVs of that type, using the CopyMapping library to dump the records
        into the raw database, preserving all fields from the original data.
        """
        self.header("Load raw FEC filings")
        name_to_model = {
            "F3X": RawF3XFiling,
            "F3P": RawF3PFiling,
            "F24": RawF24Filing,
            "sa": RawContribution,
            "se": RawIndependentExpenditure,
        }

        if options['flush']:
            self.log('Flushing the DB')
            for model in name_to_model.values():
                model.objects.all().delete()

        # Characters to strip from filenames to get filing number
        ignore = ascii_letters + punctuation
        loaded = 0

        for name, model in name_to_model.items():
            data_dir = os.path.join(self.DATA_DIR, name)
            # If, for some reason, we have no filings of this type
            if not os.path.exists(data_dir):
                continue

            # We're making a mapping of CSV to model fields for CopyMapping
            # In this case, they're identical.
            mapping = {
                field: field for field in model._meta.get_all_field_names()
            }

            for file_name in os.listdir(data_dir):
                # Hidden files will mess with us
                if not file_name.startswith('.'):
                    # Remove our list of ignore characters from the file name
                    # to give us just the filing number.
                    filing_no = int(file_name.translate(None, ignore))

                    if model.objects.filter(filing_no=filing_no).exists():
                        self.log(
                            '{} CSV for filing {} is already in the DB'.format(
                                name,
                                filing_no
                            )
                        )
                    else:
                        self.log(
                            'Loading {} CSV for filing {}'.format(
                                name,
                                filing_no
                            )
                        )
                        try:
                            c = CopyMapping(
                                model,
                                os.path.join(data_dir, file_name),
                                mapping
                            )
                            c.save(silent=True)
                            loaded += 1
                        except:
                            self.failure('Error on {} CSV for filing {}'.format(name, filing_no))

        self.success("Loaded {} CSVs".format(loaded))



from decimal import Decimal
from django.db import models
from fec_raw.fields import CharField, IntegerField, DecimalField

class RawContribution(models.Model):
    """
    One-to-one mapping of the fields in the raw data CSVs.
    Specific to Schedule A line items.
    """
    filing_no = IntegerField(
        db_index=True
    )
    form_type = CharField(
        max_length=8
    )
    filer_committee_id_number = CharField(
        max_length=9
    )
    transaction_id = CharField(
        max_length=20,
        null=True,
        blank=True
    )
    back_reference_tran_id_number = CharField(
        max_length=20,
        null=True,
        blank=True
    )
    back_reference_sched_name = CharField(
        max_length=8,
        null=True,
        blank=True
    )
    entity_type = CharField(
        max_length=3,
        null=True,
        blank=True
    )
    contributor_organization_name = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    contributor_last_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_first_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_prefix = CharField(
        max_length=10,
        null=True,
        blank=True
    )
    contributor_suffix = CharField(
        max_length=10,
        null=True,
        blank=True
    )
    contributor_street_1 = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_street_2 = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_city = CharField(
        max_length=30,
        null=True,
        blank=True
    )
    contributor_state = CharField(
        max_length=2,
        null=True,
        blank=True
    )
    contributor_zip_code = CharField(
        max_length=20,
        null=True,
        blank=True
    )
    election_code = CharField(
        max_length=5,
        null=True,
        blank=True
    )
    election_other_description = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    contribution_date = models.DateField(
        auto_now=False,
        null=True
    )
    contribution_amount = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0,
        null=True
    )
    contribution_aggregate = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0,
        null=True
    )
    contribution_purpose_descrip = CharField(
        max_length=100,
        null=True,
        blank=True
    )
    contributor_employer = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    contributor_occupation = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    donor_committee_fec_id = CharField(
        max_length=9,
        null=True,
        blank=True
    )
    donor_committee_name = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    donor_candidate_fec_id = CharField(
        max_length=9,
        null=True,
        blank=True
    )
    donor_candidate_last_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    donor_candidate_first_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    donor_candidate_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    donor_candidate_prefix = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    donor_candidate_suffix = CharField(
        max_length=10,
        null=True,
        blank=True
    )
    donor_candidate_office = CharField(
        max_length=1,
        null=True,
        blank=True
    )
    donor_candidate_state = CharField(
        max_length=2,
        null=True,
        blank=True
    )
    donor_candidate_district = CharField(
        max_length=2,
        null=True,
        blank=True
    )
    conduit_name = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    conduit_street1 = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    conduit_street2 = CharField(
        max_length=50,
        null=True,
        blank=True
    )
    conduit_city = CharField(
        max_length=30,
        null=True,
        blank=True
    )
    conduit_state = CharField(
        max_length=2,
        null=True,
        blank=True
    )
    conduit_zip_code = CharField(
        max_length=20,
        null=True,
        blank=True
    )
    memo_code = CharField(
        max_length=1,
        null=True,
        blank=True
    )
    memo_text_description = CharField(
        max_length=200,
        null=True,
        blank=True
    )
    reference_code = CharField(
        max_length=9,
        null=True,
        blank=True)

    # For lobbyist contributions from F3L SA line items
    lobbyist_registrant_organization_name = CharField(
        max_length=200,
        null=True,
        blank=True)
    lobbyist_registrant_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    lobbyist_registrant_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    lobbyist_registrant_street_1 = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_street_2 = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_city = CharField(
        max_length=30,
        null=True,
        blank=True)
    lobbyist_registrant_state = CharField(
        max_length=2,
        null=True,
        blank=True)
    lobbyist_registrant_zip_code = CharField(
        max_length=20,
        null=True,
        blank=True)
    contribution_purpose_code = CharField(
        max_length=100,
        null=True,
        blank=True)
    bundled_amount_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0,
        null=True)
    bundled_amount_semi_annual = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0,
        null=True)
    lobbyist_registrant_employer = CharField(
        max_length=50,
        null=True,
        blank=True)
    lobbyist_registrant_occupation = CharField(
        max_length=50,
        null=True,
        blank=True)
    associated_text_record = CharField(
        max_length=200,
        null=True,
        blank=True)
    memo_text = CharField(
        max_length=100,
        null=True,
        blank=True)
    reference_code = CharField(
        max_length=50,
        null=True,
        blank=True)

    def __unicode__(self):
        return 'SA {} in filing {}'.format(
            self.transaction_id,
            self.filing_no)

class RawIndependentExpenditure(models.Model):
    """
    One-to-one mapping of the fields in the raw data CSVs.
    Specific to Schedule E line items.
    """
    filing_no = models.IntegerField(
        db_index=True)
    form_type = CharField(
        max_length=8)
    filer_committee_id_number = CharField(
        max_length=9)
    transaction_id_number = CharField(
        max_length=20)
    back_reference_tran_id_number = CharField(max_length=20,
        null=True,
        blank=True)
    back_reference_sched_name = CharField(max_length=8,
        null=True,
        blank=True)
    entity_type = CharField(max_length=3,
        null=True,
        blank=True)
    payee_organization_name = CharField(
        max_length=200,
        null=True,
        blank=True)
    payee_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    payee_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    payee_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    payee_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    payee_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    payee_street_1 = CharField(
        max_length=50,
        null=True,
        blank=True)
    payee_street_2 = CharField(
        max_length=50,
        null=True,
        blank=True)
    payee_city = CharField(
        max_length=30,
        null=True,
        blank=True)
    payee_state = CharField(
        max_length=2,
        null=True,
        blank=True)
    payee_zip_code = CharField(
        max_length=20,
        null=True,
        blank=True)
    election_code = CharField(
        max_length=5,
        null=True,
        blank=True)
    election_other_description = CharField(
        max_length=20,
        null=True,
        blank=True)
    dissemination_date = models.DateField(
        auto_now=False,
        null=True)
    expenditure_amount = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    disbursement_date = models.DateField(
        auto_now=False,
        null=True)
    calendar_y_t_d_per_election_office = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    expenditure_purpose_descrip = CharField(
        max_length=100,
        null=True,
        blank=True)
    category_code = CharField(
        max_length=3,
        null=True,
        blank=True)
    payee_cmtte_fec_id_number = CharField(
        max_length=9,
        null=True,
        blank=True)
    support_oppose_code = CharField(max_length=1,
        null=True,
        blank=True)
    candidate_id_number = CharField(max_length=9,
        null=True,
        blank=True)
    candidate_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    candidate_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    candidate_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    candidate_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    candidate_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    candidate_office = CharField(
        max_length=3,
        null=True,
        blank=True)
    candidate_district = CharField(
        max_length=2,
        null=True,
        blank=True)
    candidate_state = CharField(
        max_length=2,
        null=True,
        blank=True)
    completing_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    completing_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    completing_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    completing_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    completing_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    date_signed = models.DateField(
        auto_now=False, null=True)
    memo_code = CharField(
        max_length=1,
        null=True,
        blank=True)
    memo_text_description = CharField(
        max_length=100,
        null=True,
        blank=True)

    def __unicode__(self):
        return 'IE {} in filing {}'.format(
            self.transaction_id_number,
            self.filing_no)



import os
import subprocess
import feedparser
from tomorrow import threads
from django.conf import settings
from fec_raw import get_download_directory
from fec_raw.management.commands import FecCommand



class Command(FecCommand):
    help = "Download FEC filings to data directory"
    # Path to ruby script that uses Fech to save a filing
    FECH_PATH = os.path.join(
        os.path.dirname(
            os.path.dirname(
                os.path.dirname(__file__)
            )
        ),
        'fech_filing.rb'
    )
    # If there are no downloaded filings
    DEFAULT_FIRST_FILING = 1000000
    # Where to download files
    DATA_DIR = get_download_directory()

    def add_arguments(self, parser):
        parser.add_argument(
            '--filing',
            action='store',
            nargs=1,
            type=int,
            required=False,
            help='ID of single filing to retrieve',
            dest='filing'
        )
        parser.add_argument(
            '--first',
            '-f',
            action='store',
            nargs=1,
            type=int,
            required=False,
            help='ID of first record to be added',
            dest='first_record'
        )
        parser.add_argument(
            '--last',
            '-l',
            action='store',
            nargs=1,
            type=int,
            required=False,
            help='ID of last record to be added',
            dest='last_record'
        )
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force a reload of the filings',
        )

    def handle(self, *args, **options):
        """
        Determines the first and last filing numbers to download, either
        from the provided options or from get_latest_downloaded_filing_number
        and get_latest_filing_number. Loops through those records and, if it
        hasn't been previously downloaded, downloads it.
        """
        self.header("Downloading raw FEC filings")
        # Create the raw data directory if it doesn't exist
        raw_data_path = os.path.join(self.DATA_DIR, 'raw')
        os.path.exists(raw_data_path) or os.makedirs(raw_data_path)

        # Figure out which filings we're going to try to get
        # If we want a specific filing, set both the first and last record to
        # that filing
        if options['filing']:
            options['first_record'] = options['filing']
            options['last_record'] = options['filing']

        if options['first_record']:
            first_record = options['first_record'][0]
        else:
            first_record = self.get_latest_downloaded_filing_number() + 1

        if options['last_record']:
            last_record = options['last_record'][0]
        else:
            last_record = self.get_latest_filing_number()

        if first_record > last_record:
            self.failure('No records to download')
            return

        self.log(
            'Attempting to download filings {} to {}'.format(
                first_record,
                last_record
            )
        )
        # Loop through all filings we're interested in
        for filing_no in range(first_record, last_record + 1):
            # Create string version of the filing number
            filing_id = str(filing_no)

            # It should exist at this path
            filing_path = os.path.join(
                self.DATA_DIR,
                'raw',
                filing_id + '.fec'
            )

            # If it does AND we're not forcing a refresh ...
            if os.path.isfile(filing_path) and not options['force']:
                # Skip to the next record
                self.log('Already downloaded filing {}'.format(filing_id))
                continue
            # Otherwise ...
            else:
                # Interface with fech to get the data
                self.fech_filing(filing_id)

    @threads(getattr(settings, 'FEC_DOWNLOAD_THREADS', 4))
    def fech_filing(self, filing_no):
        """
        Interfaces with Ruby library Fech to return data for a particular filing
        number, saves data to CSVs in DATA_DIR
        """
        self.log('Fech-ing filing {}'.format(filing_no))

        p = subprocess.Popen(
            ['ruby', self.FECH_PATH, filing_no, self.DATA_DIR],
            stdout=subprocess.PIPE,
            stderr=None
        )

        # start the subprocess
        output = p.communicate()
        # this is the stdout of the ruby script
        message = output[0]

        # TODO: Refactor/fix the way this error handling works
        if message[0] == 'E':
            self.failure(' - Failed to download filing {}'.format(filing_no))
        elif message[0] == 'S':
            self.success(' - Downloaded filing {}'.format(filing_no))

    def get_latest_downloaded_filing_number(self):
        """
        Checks data directory to get latest previously downloaded
        filing number.
        """
        files = os.listdir(os.path.join(self.DATA_DIR, 'raw'))
        try:
            filing_numbers = [int(filename.split('.')[0]) for filename in files if not (filename.startswith('.') or filename.startswith('fech'))]
            return sorted(filing_numbers, reverse=True)[0]
        except:
            return self.DEFAULT_FIRST_FILING

    def get_latest_filing_number(self):
        """
        Uses FEC RSS feed to get the ID of the latest filing.
        """
        self.log('Getting latest filing number from FEC...')

        url = 'http://efilingapps.fec.gov/rss/generate?preDefinedFilingType=ALL'
        d = feedparser.parse(url)
        # Sorted entries by date, most recent first
        entries = sorted(
            d.entries,
            key=lambda entry: entry['published_parsed'],
            reverse=True
        )
        # Get filing ID from link that looks like:
        # http://docquery.fec.gov/dcdev/posted/1020510.fec
        link = entries[0]['link']
        latest = int(link.split('/')[-1].replace('.fec', ''))

        self.log('Latest filing number is {}'.format(latest))
        return latest



from django.utils.termcolors import colorize
from django.core.management.base import BaseCommand


class FecCommand(BaseCommand):

    def header(self, string):
        self.stdout.write(
            colorize(string, fg="cyan", opts=("bold",))
        )

    def log(self, string):
        self.stdout.write(
            colorize("%s" % string, fg="white")
        )

    def success(self, string):
        self.stdout.write(
            colorize(string, fg="green")
        )

    def failure(self, string):
        self.stdout.write(
            colorize(string, fg="red")
        )



from django.contrib import admin
from fec_raw.models import RawF3XFiling, RawF3PFiling, RawF24Filing
from fec_raw.models import RawContribution, RawIndependentExpenditure

class RawF3XFilingAdmin(admin.ModelAdmin):
    pass

class RawF3PFilingAdmin(admin.ModelAdmin):
    pass

class RawF24FilingAdmin(admin.ModelAdmin):
    pass

class RawContributionAdmin(admin.ModelAdmin):
    pass

class RawIndependentExpenditureAdmin(admin.ModelAdmin):
    pass

admin.site.register(RawF3XFiling, RawF3XFilingAdmin)
admin.site.register(RawF3PFiling, RawF3PFilingAdmin)
admin.site.register(RawF24Filing, RawF24FilingAdmin)
admin.site.register(RawContribution, RawContributionAdmin)
admin.site.register(RawIndependentExpenditure, RawIndependentExpenditureAdmin)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup


setup(
    name='django-fec-raw-data',
    version='0.0.3',
    packages=(
        'fec_raw',
        'fec_raw.management',
        'fec_raw.management.commands',
        'fec_raw.models',
    ),
    include_package_data=True,
    license='MIT',
    description='A Django app to download, extract and load campaign finance from the Federal Election Commission',
    author='The Los Angeles Times Data Desk',
    author_email='datadesk@latimes.com',
    classifiers=(
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
    install_requires=(
        'django-postgres-copy>=0.0.6',
        'feedparser>=5.2.1',
        'tomorrow>=0.2.3',
    )
)



from decimal import Decimal
from django.db import models
from fec_raw.fields import CharField, IntegerField, DecimalField

class RawF3XFiling(models.Model):
    """
    One-to-one mapping of the fields in the raw data CSVs.
    Specific to F3X filing topsheets.
    """
    filing_no = IntegerField(
        unique=True)
    # ID of filing that this one amends
    report_id = CharField(
        max_length=20,
        null=True,
        blank=True)
    # 0-indexed (1 means first amendment)
    report_number = IntegerField(
        default=0)

    form_type = CharField(
        max_length=4,
        null=True,
        blank=True)
    filer_committee_id_number = CharField(
        max_length=9)

    committee_name = CharField(
        max_length=200,
        null=True,
        blank=True)
    change_of_address = CharField(
        max_length=1,
        null=True,
        blank=True)
    street_1 = CharField(
        max_length=50,
        null=True,
        blank=True)
    street_2 = CharField(
        max_length=50,
        null=True,
        blank=True)
    city = CharField(
        max_length=30,
        null=True,
        blank=True)
    state = CharField(
        max_length=2,
        null=True,
        blank=True)
    zip_code = CharField(
        max_length=20,
        null=True,
        blank=True)

    report_code = CharField(
        max_length=3,
        null=True,
        blank=True)
    election_code = CharField(
        max_length=5,
        null=True,
        blank=True)
    date_of_election = models.DateField(
        auto_now=False,
        null=True,
        blank=True)
    state_of_election = CharField(
        max_length=2,
        null=True,
        blank=True)
    coverage_from_date = models.DateField(
        auto_now=False,
        null=True)
    coverage_through_date = models.DateField(
        auto_now=False,
        null=True)

    qualified_committee = CharField(
        max_length=1,
        null=True,
        blank=True)

    treasurer_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    treasurer_suffix = CharField(
        max_length=200,
        null=True,
        blank=True)
    date_signed = models.DateField(
        auto_now=False)
    
    col_a_cash_on_hand_beginning_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_subtotal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_cash_on_hand_close_of_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_debts_to = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_debts_by = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individuals_itemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individuals_unitemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individual_contribution_total = DecimalField(
        max_digits=16,
        decimal_places=2, 
        default=0)
    col_a_political_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_political_committees_pacs = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_transfers_from_aff_other_party_cmttees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_loan_repayments_received = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_offsets_to_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_contributions_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_federal_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_transfers_from_nonfederal_h3 = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_levin_funds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_nonfederal_transfers = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_federal_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_shared_operating_expenditures_federal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_shared_operating_expenditures_nonfederal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_federal_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_transfers_to_affiliated = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_contributions_to_candidates = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_independent_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_coordinated_expenditures_by_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_loan_repayments_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_loans_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_refunds_to_individuals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_refunds_to_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_refunds_to_other_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_federal_election_activity_federal_share = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_federal_election_activity_levin_share = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_federal_election_activity_all_federal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_federal_election_activity_total = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_federal_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_net_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_federal_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_offsets_to_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_net_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_cash_on_hand_jan_1 = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_year = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_subtotal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_cash_on_hand_close_of_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individuals_itemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individuals_unitemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individual_contribution_total = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_political_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_political_committees_pacs = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_transfers_from_aff_other_party_cmttees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_loan_repayments_received = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_offsets_to_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_contributions_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_federal_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_transfers_from_nonfederal_h3 = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_levin_funds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_nonfederal_transfers = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_federal_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_shared_operating_expenditures_federal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_shared_operating_expenditures_nonfederal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_federal_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_transfers_to_affiliated = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_contributions_to_candidates = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_independent_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_coordinated_expenditures_by_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_loan_repayments_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_loans_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_refunds_to_individuals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_refunds_to_party_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_refunds_to_other_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_federal_election_activity_federal_share = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_federal_election_activity_levin_share = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_federal_election_activity_all_federal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_federal_election_activity_total = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_federal_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_net_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_federal_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_offsets_to_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_net_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)

    def __unicode__(self):
        return 'F3X filing {}'.format(self.filing_no)

class RawF3PFiling(models.Model):
    """
    Contains one-to-one copy of the raw fields produced by FECh.
    Specific to F3P filings.
    """
    filing_no = IntegerField(
        unique=True)
    # ID of filing that this one amends
    report_id = CharField(
        max_length=20,
        null=True,
        blank=True)
    # 0-indexed (1 means first amendment)
    report_number = IntegerField(
        default=0)

    form_type = CharField(
        max_length=4,
        null=True,
        blank=True)
    filer_committee_id_number = CharField(
        max_length=9, 
        null=True,
        blank=True)

    committee_name = CharField(
        max_length=200,
        null=True,
        blank=True)
    change_of_address = CharField(
        max_length=1,
        null=True,
        blank=True)
    street_1 = CharField(
        max_length=50,
        null=True,
        blank=True)
    street_2 = CharField(
        max_length=50,
        null=True,
        blank=True)
    city = CharField(
        max_length=30,
        null=True,
        blank=True)
    state = CharField(
        max_length=2,
        null=True,
        blank=True)
    zip_code = CharField(
        max_length=20, 
        null=True, 
        blank=True)

    activity_primary = CharField(
        max_length=1,
        null=True,
        blank=True)
    activity_general = CharField(
        max_length=1,
        null=True,
        blank=True)
    report_code = CharField(
        max_length=3,
        null=True,
        blank=True)
    election_code = CharField(
        max_length=5,
        null=True,
        blank=True)
    date_of_election = models.DateField(
        auto_now=False, 
        null=True, 
        blank=True)
    state_of_election = CharField(
        max_length=2,
        null=True,
        blank=True)
    coverage_from_date = models.DateField(
        auto_now=False,
        null=True)
    coverage_through_date = models.DateField(
        auto_now=False,
        null=True)

    treasurer_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    treasurer_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    date_signed = models.DateField(
        auto_now=False)

    col_a_cash_on_hand_beginning_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_subtotal = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_cash_on_hand_close_of_period = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_debts_to = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_debts_by = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_expenditures_subject_to_limits = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_net_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_net_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_federal_funds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individuals_itemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individuals_unitemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individual_contribution_total = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_political_party_committees_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_political_committees_pacs = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_the_candidate = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_contributions = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_transfers_from_aff_other_party_cmttees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_received_from_or_guaranteed_by_cand = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_operating = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_fundraising = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_legal_and_accounting = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_offsets_to_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_transfers_to_other_authorized_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_fundraising_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_exempt_legal_accounting_disbursement = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_made_or_guaranteed_by_candidate = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_repayments = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_loan_repayments_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_individuals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_political_party_committees_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_political_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_total_contributions_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_other_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_items_on_hand_to_be_liquidated = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_alabama = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_alaska = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_arizona = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_arkansas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_california = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_colorado = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_connecticut = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_delaware = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_dist_of_columbia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_florida = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_georgia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_hawaii = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_idaho = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_illinois = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_indiana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_iowa = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_kansas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_kentucky = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_louisiana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_maine = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_maryland = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_massachusetts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_michigan = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_minnesota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_mississippi = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_missouri = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_montana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_nebraska = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_nevada = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_new_hampshire = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_new_jersey = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_new_mexico = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_new_york = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_north_carolina = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_north_dakota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_ohio = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_oklahoma = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_oregon = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_pennsylvania = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_rhode_island = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_south_carolina = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_south_dakota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_tennessee = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_texas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_utah = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_vermont = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_virginia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_washington = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_west_virginia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_wisconsin = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_wyoming = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_puerto_rico = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_guam = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_virgin_islands = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_a_totals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)

    col_b_federal_funds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individuals_itemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individuals_unitemized = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individual_contribution_total = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_political_party_committees_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_political_committees_pacs = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_the_candidate = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_contributions_other_than_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_transfers_from_aff_other_party_cmttees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_received_from_or_guaranteed_by_cand = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_loans = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_operating = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_fundraising = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_legal_and_accounting = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_offsets_to_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_receipts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_operating_expenditures = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_transfers_to_other_authorized_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_fundraising_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_exempt_legal_accounting_disbursement = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_made_or_guaranteed_by_the_candidate = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_repayments = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_loan_repayments_made = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_individuals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_political_party_committees_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_political_committees = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_contributions_refunds = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_other_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_total_disbursements = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_alabama = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_alaska = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_arizona = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_arkansas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_california = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_colorado = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_connecticut = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_delaware = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_dist_of_columbia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_florida = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_georgia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_hawaii = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_idaho = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_illinois = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_indiana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_iowa = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_kansas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_kentucky = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_louisiana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_maine = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_maryland = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_massachusetts = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_michigan = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_minnesota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_mississippi = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_missouri = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_montana = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_nebraska = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_nevada = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_new_hampshire = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_new_jersey = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_new_mexico = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_new_york = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_north_carolina = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_north_dakota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_ohio = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_oklahoma = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_oregon = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_pennsylvania = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_rhode_island = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_south_carolina = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_south_dakota = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_tennessee = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_texas = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_utah = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_vermont = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_virginia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_washington = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_west_virginia = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_wisconsin = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_wyoming = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_puerto_rico = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_guam = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_virgin_islands = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)
    col_b_totals = DecimalField(
        max_digits=16,
        decimal_places=2,
        default=0)

    def __unicode__(self):
        return 'F3P filing {}'.format(self.filing_no)

class RawF24Filing(models.Model):
    """
    One-to-one mapping of the fields in the raw data CSVs.
    Specific to F24 filing topsheets.
    """
    filing_no = IntegerField(
        unique=True)
    # ID of filing that this one amends
    report_id = CharField(
        max_length=20,
        null=True,
        blank=True)
    # 0-indexed (1 means first amendment)
    report_number = IntegerField(
        default=0)

    form_type = CharField(
        max_length=4,
        null=True,
        blank=True)
    filer_committee_id_number = CharField(
        max_length=9,
        null=True,
        blank=True)
    report_type = CharField(
        max_length=2,
        null=True,
        blank=True)
    original_amendment_date = models.DateField(
        auto_now=False,
        null=True)

    committee_name = CharField(
        max_length=200,
        null=True,
        blank=True)
    street_1 = CharField(
        max_length=50,
        null=True,
        blank=True)
    street_2 = CharField(
        max_length=50,
        null=True,
        blank=True)
    city = CharField(
        max_length=30,
        null=True,
        blank=True)
    state = CharField(
        max_length=2,
        null=True,
        blank=True)
    zip_code = CharField(
        max_length=20,
        null=True,
        blank=True)

    treasurer_last_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_first_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_middle_name = CharField(
        max_length=50,
        null=True,
        blank=True)
    treasurer_prefix = CharField(
        max_length=10,
        null=True,
        blank=True)
    treasurer_suffix = CharField(
        max_length=10,
        null=True,
        blank=True)
    date_signed = models.DateField(
        auto_now=False)

    def __unicode__(self):
        return 'F24 filing {}'.format(self.filing_no)



"""
Custom field overrides that allow for cleaning and transforming the data
when it is bulk loaded into the database with PostgreSQL's COPY command via
django-postgres-copy.
"""
from django.db.models import fields

class DecimalField(fields.DecimalField):
    copy_type = 'text'
    copy_template = """
    CASE
        WHEN "%(name)s" IS NULL
            THEN 0
        WHEN "%(name)s" IS NOT NULL
            THEN CAST("%(name)s" AS decimal(16,2))
    END
    """

class IntegerField(fields.IntegerField):
    copy_type = 'text'
    copy_template = """
    CASE
        WHEN "%(name)s" IS NULL
            THEN 0
        WHEN "%(name)s" IS NOT NULL
            THEN CAST("%(name)s" AS INTEGER)
    END
    """

class CharField(fields.CharField):
    copy_type = 'text'
    copy_template = """
    CASE
        WHEN "%(name)s" IS NULL
            THEN ''
        ELSE UPPER(TRIM(BOTH ' ' from %(name)s))
    END
    """